import org.junit.jupiter.api.Test;
import org.mikgul.Coordinates;
import org.mikgul.WeatherEntry;
import org.mikgul.WeatherJsonParser;
import org.mikgul.WeatherResponse;

import java.io.IOException;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;


class WeatherAppTest {
    private boolean checkLongitude(double longitude) {
        return longitude <= 180.00 && longitude >= -180.00;
    }

    private boolean checkLatitude(double latitude) {
        return latitude <= 90.00 && latitude >= -90.00;
    }

    private WeatherResponse goodResponse(String body) {
        WeatherResponse res = new WeatherResponse();
        res.setStatusCode(200);
        res.setResponseBody(body);
        return res;
    }

    private WeatherResponse badResponse(String message) {
        WeatherResponse res = new WeatherResponse();
        res.setStatusCode(400);
        res.setResponseBody(message);
        return res;
    }

    // ********** TESTY Z POZYTYWNYM SCENARIUSZEM **********

    @Test
    void testWeatherRequestWithGoodCoordinates() throws IOException {
        Coordinates goodCords = new Coordinates(40.7128, -74.0060);

        assertTrue(checkLongitude(goodCords.longitude()));
        assertTrue(checkLatitude(goodCords.latitude()));

        String sampleBody = "{\"weather\":[{\"description\":\"pochmurnie\"}],\"main\":{\"temp\":25.5,\"feels_like\":25.31,\"temp_min\":19.58,\"temp_max\":28.12,\"pressure\":1015,\"humidity\":50}}";
        WeatherResponse mockResponse = mock(WeatherResponse.class);
        when(mockResponse.sendWeatherRequest(any()))
                .thenReturn(goodResponse(sampleBody));

        WeatherResponse realResponse = mockResponse.sendWeatherRequest(goodCords);

        assertEquals(200, realResponse.getStatusCode());

        WeatherJsonParser parser = new WeatherJsonParser();
        WeatherEntry result = parser.parseWeatherData("Nowy Jork", realResponse);

        assertEquals("Nowy Jork", result.getMiasto());
        assertEquals("pochmurnie", result.getPogoda());
        assertEquals(25.5, result.getTemperatura());
        assertEquals(1015, result.getCisnienie());
        assertEquals(50, result.getWilgotnosc());
    }

    @Test
    void testWeatherRequestParsing() throws IOException {
        // zakladamy ze zapytanie bylo poprawne a odpowiedz pozytywna, wiec nie sprawdzamy ich 'assertami'
        Coordinates cords = new Coordinates(35.6895, 139.6917);

        String sampleBody = "{\"weather\":[{\"description\":\"słonecznie\"}],\"main\":{\"temp\":13.5,\"feels_like\":11.21,\"temp_min\":10.58,\"temp_max\":16.12,\"pressure\":1022,\"humidity\":23}}";
        WeatherResponse mockResponse = mock(WeatherResponse.class);
        when(mockResponse.sendWeatherRequest(any()))
                .thenReturn(goodResponse(sampleBody));

        WeatherJsonParser parser = new WeatherJsonParser();
        WeatherEntry result = parser.parseWeatherData("Tokio", mockResponse.sendWeatherRequest(cords));

        assertEquals("Tokio", result.getMiasto());
        assertEquals("słonecznie", result.getPogoda());
        assertEquals(13.5, result.getTemperatura());
        assertEquals(1022, result.getCisnienie());
        assertEquals(23, result.getWilgotnosc());

        assertEquals("Pogoda w Tokio:\nsłonecznie, temperatura: 13,50 C; cisnienie: 1022 hPa; wilgotnosc: 23%\n",
                result.toString());
    }

    // ********** TESTY Z NEGATYWNYM SCENARIUSZEM **********

    @Test
    void testWeatherRequestWithBadLongitude() {
        Coordinates badCords = new Coordinates(40.7128, -192.0060);

        assertFalse(checkLongitude(badCords.longitude()));
        assertTrue(checkLatitude(badCords.latitude()));

        String message = "{\"cod\":\"400\",\"message\":\"wrong longitude\"}";
        WeatherResponse mockResponse = mock(WeatherResponse.class);
        when(mockResponse.sendWeatherRequest(any()))
                .thenReturn(badResponse(message));

        WeatherResponse realResponse = mockResponse.sendWeatherRequest(badCords);

        assertEquals(400, realResponse.getStatusCode());
        assertEquals("{\"cod\":\"400\",\"message\":\"wrong longitude\"}", realResponse.getBody());
    }

    @Test
    void testWeatherRequestWithBadLatitude() {
        Coordinates badCords = new Coordinates(90.7128, -74.0060);

        assertTrue(checkLongitude(badCords.longitude()));
        assertFalse(checkLatitude(badCords.latitude()));

        String message = "{\"cod\":\"400\",\"message\":\"wrong latitude\"}";
        WeatherResponse mockResponse = mock(WeatherResponse.class);
        when(mockResponse.sendWeatherRequest(any()))
                .thenReturn(badResponse(message));

        WeatherResponse realResponse = mockResponse.sendWeatherRequest(badCords);

        assertEquals(400, realResponse.getStatusCode());
        assertEquals("{\"cod\":\"400\",\"message\":\"wrong latitude\"}", realResponse.getBody());
    }
}